var __accessCheck = (obj, member, msg) => {
  if (!member.has(obj))
    throw TypeError("Cannot " + msg);
};
var __privateGet = (obj, member, getter) => {
  __accessCheck(obj, member, "read from private field");
  return getter ? getter.call(obj) : member.get(obj);
};
var __privateAdd = (obj, member, value) => {
  if (member.has(obj))
    throw TypeError("Cannot add the same private member more than once");
  member instanceof WeakSet ? member.add(obj) : member.set(obj, value);
};
var __privateSet = (obj, member, value, setter) => {
  __accessCheck(obj, member, "write to private field");
  setter ? setter.call(obj, value) : member.set(obj, value);
  return value;
};
var __privateMethod = (obj, member, method) => {
  __accessCheck(obj, member, "access private method");
  return method;
};
var _diagnosticsChecker, _shouldWatch, _syncCli, _settings, _logging, _fs, _watcher, _filesCount, _updateDiagnostics, _openDocuments, openDocuments_fn, _checkAllFiles, checkAllFiles_fn, _checkForDiagnostics, checkForDiagnostics_fn, _watch, watch_fn, _addDocument, addDocument_fn, _logDiagnosticsSeverity, logDiagnosticsSeverity_fn, _breakDownDiagnostics, breakDownDiagnostics_fn;
import { AstroCheck, DiagnosticSeverity } from "@astrojs/language-server";
import glob from "fast-glob";
import * as fs from "fs";
import { bold, dim, red, yellow } from "kleur/colors";
import { createRequire } from "module";
import ora from "ora";
import { fileURLToPath, pathToFileURL } from "url";
import { printDiagnostic } from "./print.js";
import { debug, info } from "../../core/logger/core.js";
import { join } from "node:path";
var CheckResult = /* @__PURE__ */ ((CheckResult2) => {
  CheckResult2[CheckResult2["ExitWithSuccess"] = 0] = "ExitWithSuccess";
  CheckResult2[CheckResult2["ExitWithError"] = 1] = "ExitWithError";
  CheckResult2[CheckResult2["Listen"] = 2] = "Listen";
  return CheckResult2;
})(CheckResult || {});
const ASTRO_GLOB_PATTERN = "**/*.astro";
async function check(settings, { logging, flags }) {
  const checkFlags = parseFlags(flags);
  if (checkFlags.watch) {
    info(logging, "check", "Checking files in watch mode");
  } else {
    info(logging, "check", "Checking files");
  }
  const { syncCli } = await import("../../core/sync/index.js");
  const root = settings.config.root;
  const require2 = createRequire(import.meta.url);
  const diagnosticChecker = new AstroCheck(
    root.toString(),
    require2.resolve("typescript/lib/tsserverlibrary.js", {
      paths: [root.toString()]
    })
  );
  return new AstroChecker({
    syncCli,
    settings,
    fileSystem: fs,
    logging,
    diagnosticChecker,
    isWatchMode: checkFlags.watch
  });
}
class AstroChecker {
  constructor({
    diagnosticChecker,
    isWatchMode,
    syncCli,
    settings,
    fileSystem,
    logging
  }) {
    __privateAdd(this, _openDocuments);
    __privateAdd(this, _checkAllFiles);
    __privateAdd(this, _checkForDiagnostics);
    __privateAdd(this, _watch);
    __privateAdd(this, _addDocument);
    __privateAdd(this, _logDiagnosticsSeverity);
    __privateAdd(this, _breakDownDiagnostics);
    __privateAdd(this, _diagnosticsChecker, void 0);
    __privateAdd(this, _shouldWatch, void 0);
    __privateAdd(this, _syncCli, void 0);
    __privateAdd(this, _settings, void 0);
    __privateAdd(this, _logging, void 0);
    __privateAdd(this, _fs, void 0);
    __privateAdd(this, _watcher, void 0);
    __privateAdd(this, _filesCount, void 0);
    __privateAdd(this, _updateDiagnostics, void 0);
    __privateSet(this, _diagnosticsChecker, diagnosticChecker);
    __privateSet(this, _shouldWatch, isWatchMode);
    __privateSet(this, _syncCli, syncCli);
    __privateSet(this, _logging, logging);
    __privateSet(this, _settings, settings);
    __privateSet(this, _fs, fileSystem);
    __privateSet(this, _filesCount, 0);
  }
  async check() {
    return await __privateMethod(this, _checkAllFiles, checkAllFiles_fn).call(this, true);
  }
  async watch() {
    await __privateMethod(this, _checkAllFiles, checkAllFiles_fn).call(this, true);
    await __privateMethod(this, _watch, watch_fn).call(this);
    return 2 /* Listen */;
  }
  async stop() {
    var _a;
    await ((_a = __privateGet(this, _watcher)) == null ? void 0 : _a.close());
  }
  get isWatchMode() {
    return __privateGet(this, _shouldWatch);
  }
}
_diagnosticsChecker = new WeakMap();
_shouldWatch = new WeakMap();
_syncCli = new WeakMap();
_settings = new WeakMap();
_logging = new WeakMap();
_fs = new WeakMap();
_watcher = new WeakMap();
_filesCount = new WeakMap();
_updateDiagnostics = new WeakMap();
_openDocuments = new WeakSet();
openDocuments_fn = async function() {
  __privateSet(this, _filesCount, await openAllDocuments(
    __privateGet(this, _settings).config.root,
    [],
    __privateGet(this, _diagnosticsChecker)
  ));
};
_checkAllFiles = new WeakSet();
checkAllFiles_fn = async function(openDocuments) {
  const processExit = await __privateGet(this, _syncCli).call(this, __privateGet(this, _settings), {
    logging: __privateGet(this, _logging),
    fs: __privateGet(this, _fs)
  });
  if (processExit === 1)
    return processExit;
  let spinner = ora(
    ` Getting diagnostics for Astro files in ${fileURLToPath(__privateGet(this, _settings).config.root)}\u2026`
  ).start();
  if (openDocuments) {
    await __privateMethod(this, _openDocuments, openDocuments_fn).call(this);
  }
  let diagnostics = await __privateGet(this, _diagnosticsChecker).getDiagnostics();
  spinner.succeed();
  let brokenDownDiagnostics = __privateMethod(this, _breakDownDiagnostics, breakDownDiagnostics_fn).call(this, diagnostics);
  __privateMethod(this, _logDiagnosticsSeverity, logDiagnosticsSeverity_fn).call(this, brokenDownDiagnostics);
  return brokenDownDiagnostics.errors > 0 ? 1 /* ExitWithError */ : 0 /* ExitWithSuccess */;
};
_checkForDiagnostics = new WeakSet();
checkForDiagnostics_fn = function() {
  clearTimeout(__privateGet(this, _updateDiagnostics));
  __privateSet(this, _updateDiagnostics, setTimeout(async () => await __privateMethod(this, _checkAllFiles, checkAllFiles_fn).call(this, false), 500));
};
_watch = new WeakSet();
watch_fn = async function() {
  const { default: chokidar } = await import("chokidar");
  __privateSet(this, _watcher, chokidar.watch(
    join(fileURLToPath(__privateGet(this, _settings).config.root), ASTRO_GLOB_PATTERN),
    {
      ignored: ["**/node_modules/**"],
      ignoreInitial: true
    }
  ));
  __privateGet(this, _watcher).on("add", (file) => {
    __privateMethod(this, _addDocument, addDocument_fn).call(this, file);
    __privateSet(this, _filesCount, __privateGet(this, _filesCount) + 1);
    __privateMethod(this, _checkForDiagnostics, checkForDiagnostics_fn).call(this);
  });
  __privateGet(this, _watcher).on("change", (file) => {
    __privateMethod(this, _addDocument, addDocument_fn).call(this, file);
    __privateMethod(this, _checkForDiagnostics, checkForDiagnostics_fn).call(this);
  });
  __privateGet(this, _watcher).on("unlink", (file) => {
    __privateGet(this, _diagnosticsChecker).removeDocument(file);
    __privateSet(this, _filesCount, __privateGet(this, _filesCount) - 1);
    __privateMethod(this, _checkForDiagnostics, checkForDiagnostics_fn).call(this);
  });
};
_addDocument = new WeakSet();
addDocument_fn = function(filePath) {
  const text = fs.readFileSync(filePath, "utf-8");
  __privateGet(this, _diagnosticsChecker).upsertDocument({
    uri: pathToFileURL(filePath).toString(),
    text
  });
};
_logDiagnosticsSeverity = new WeakSet();
logDiagnosticsSeverity_fn = function(result) {
  info(
    __privateGet(this, _logging),
    "diagnostics",
    [
      bold(`Result (${__privateGet(this, _filesCount)} file${__privateGet(this, _filesCount) === 1 ? "" : "s"}): `),
      bold(red(`${result.errors} ${result.errors === 1 ? "error" : "errors"}`)),
      bold(yellow(`${result.warnings} ${result.warnings === 1 ? "warning" : "warnings"}`)),
      dim(`${result.hints} ${result.hints === 1 ? "hint" : "hints"}
`)
    ].join(`
${dim("-")} `)
  );
};
_breakDownDiagnostics = new WeakSet();
breakDownDiagnostics_fn = function(diagnostics) {
  let result = {
    errors: 0,
    warnings: 0,
    hints: 0
  };
  diagnostics.forEach((diag) => {
    diag.diagnostics.forEach((d) => {
      info(__privateGet(this, _logging), "diagnostics", `
 ${printDiagnostic(diag.fileUri, diag.text, d)}`);
      switch (d.severity) {
        case DiagnosticSeverity.Error: {
          result.errors++;
          break;
        }
        case DiagnosticSeverity.Warning: {
          result.warnings++;
          break;
        }
        case DiagnosticSeverity.Hint: {
          result.hints++;
          break;
        }
      }
    });
  });
  return result;
};
async function openAllDocuments(workspaceUri, filePathsToIgnore, checker) {
  const files = await glob(ASTRO_GLOB_PATTERN, {
    cwd: fileURLToPath(workspaceUri),
    ignore: ["node_modules/**"].concat(filePathsToIgnore.map((ignore) => `${ignore}/**`)),
    absolute: true
  });
  for (const file of files) {
    debug("check", `Adding file ${file} to the list of files to check.`);
    const text = fs.readFileSync(file, "utf-8");
    checker.upsertDocument({
      uri: pathToFileURL(file).toString(),
      text
    });
  }
  return files.length;
}
function parseFlags(flags) {
  return {
    watch: flags.watch ?? false
  };
}
export {
  AstroChecker,
  CheckResult,
  check
};
